/* siriuswait.c -- monitoring the Sirius Calendar Service
 * Author: David Eisenstat (deisenst@cs.princeton.edu)
 *
 * To compile:
 * $ gcc -o siriuswait siriuswait.c
 *
 * 07/04/17: Initial version
 */

#include <err.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>

static void usage(void) {
  errx(1, "Usage: siriuswait [-n] <slicename>");
}

int main(int argc, char *argv[], char *envp[]) {
  int waitforacquire = 1;  /* 0 if the -n option is used */
  char *slicename;  /* name of the slice we're waiting for */
  struct addrinfo hints;  /* these three are used for getaddrinfo() */
  struct addrinfo *res = NULL;
  int rc;
  int s;  /* the socket */
  char buf[1024];  /* buffer space */
  size_t len = 0;  /* amount of buffer occupied */
  ssize_t readamt;
  char *eol;  /* pointer to the end of the line */

  /* decode the command line */
  if(argc!=2 && argc!=3) {
    usage();
  } else if(3 == argc) {
    if(!strcmp("-n", argv[1])) {
      waitforacquire = 0;
    } else {
      usage();
    }
  }
  slicename = argv[argc-1];

  /* look up localhost:8124 */
  memset(&hints, 0, sizeof hints);
  hints.ai_family = AF_UNSPEC;
  hints.ai_socktype = SOCK_STREAM;
  hints.ai_protocol = IPPROTO_TCP;
  if((rc = getaddrinfo(NULL, "8124", &hints, &res)) != 0) {
    errx(1, "getaddrinfo(): %s", gai_strerror(rc));
  }

  /* connect to Sirius */
  if((s = socket(res->ai_family, res->ai_socktype, res->ai_protocol)) == -1) {
    err(1, "socket()");
  }
  if(connect(s, res->ai_addr, res->ai_addrlen)) {
    err(1, "connect()");
  }
  shutdown(s, SHUT_WR);

  /* wait for the event to occur */
  for(;;) {
    readamt = recv(s, buf+len, sizeof buf - len, 0);
    if(readamt > 0) {
      len += readamt;
    } else if(-1 == readamt) {
      err(1, "recv()");
    } else {
      errx(1, "Connection unexpectedly closed");
    }

    for(;;) {
      eol = (char *)memchr(buf, '\n', len);
      if(!eol) break;
      /* make buf into a string temporarily */
      *eol = '\0';
      /* I wish there were a logical XOR operation */
      if(!strcmp(slicename, buf)) {
	if(waitforacquire) {
	  goto finish;
	}
      } else {
	if(!waitforacquire) {
	  goto finish;
	}
      }
      /* remove the line from buf */
      memmove(buf, eol+1, len - ((eol+1) - buf));
      len -= (eol+1) - buf;
    }
  }

 finish:
  shutdown(s, SHUT_RD);
  close(s);
  freeaddrinfo(res);

  return 0;
}

